from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from .common import _BulkAdExtensionBase
from .common import _BulkAdGroupAdExtensionAssociation
from .common import _BulkCampaignAdExtensionAssociation
from .common import _BulkAccountAdExtensionAssociation

from bingads.v13.internal.extensions import *


_ActionAdExtension = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('ActionAdExtension'))


class BulkActionAdExtension(_BulkAdExtensionBase):
    """ Represents a action ad extension.

    This class exposes the :attr:`action_ad_extension` property that can be read and written
    as fields of the Action Ad Extension record in a bulk file.

    For more information, see Action Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, ad_extension=None):
        if ad_extension and not isinstance(ad_extension, _ActionAdExtension):
            raise ValueError('The type of ad_extension is: {0}, should be: {1}'.format(
                type(ad_extension),
                'ActionAdExtension'
            ))
        super(BulkActionAdExtension, self).__init__(
            account_id=account_id,
            ad_extension=ad_extension
        )
        self._action_text = None
        

    @property
    def action_ad_extension(self):
        """ The action ad extension.

        see Action Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad_extension

    @action_ad_extension.setter
    def action_ad_extension(self, value):
        self._ad_extension = value
        
        
    @property
    def action_text(self):
        """ The action text.

        """

        return self._action_text

    @action_text.setter
    def action_text(self, value):
        self._action_text = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.ActionType,
            field_to_csv=lambda c: bulk_str(c.action_ad_extension.ActionType),
            csv_to_field=lambda c, v: setattr(c.action_ad_extension, 'ActionType', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.action_ad_extension.FinalUrls, c.action_ad_extension.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.action_ad_extension.FinalUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalMobileUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.action_ad_extension.FinalMobileUrls, c.action_ad_extension.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.action_ad_extension.FinalMobileUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TrackingTemplate,
            field_to_csv=lambda c: bulk_optional_str(c.action_ad_extension.TrackingUrlTemplate, c.action_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.action_ad_extension, 'TrackingUrlTemplate', v if v else '')
        ),
        _SimpleBulkMapping(
            header=_StringTable.Language,
            field_to_csv=lambda c: bulk_optional_str(c.action_ad_extension.Language, c.action_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.action_ad_extension, 'Language', v if v else '')
        ),
        _SimpleBulkMapping(
            header=_StringTable.CustomParameter,
            field_to_csv=lambda c: field_to_csv_UrlCustomParameters(c.action_ad_extension),
            csv_to_field=lambda c, v: csv_to_field_UrlCustomParameters(c.action_ad_extension, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ActionText,
            field_to_csv=lambda c: c.action_text,
            csv_to_field=lambda c, v: setattr(c, 'action_text', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrlSuffix,
            field_to_csv=lambda c: bulk_optional_str(c.action_ad_extension.FinalUrlSuffix, c.action_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.action_ad_extension, 'FinalUrlSuffix', v)
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        self.action_ad_extension = _CAMPAIGN_OBJECT_FACTORY_V13.create('ActionAdExtension')
        self.action_ad_extension.Type = 'ActionAdExtension'
        super(BulkActionAdExtension, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkActionAdExtension._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.action_ad_extension, 'action_ad_extension')
        super(BulkActionAdExtension, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkActionAdExtension._MAPPINGS)


class BulkAccountActionAdExtension(_BulkAccountAdExtensionAssociation):
    """ Represents an account level action ad extension.

    This class exposes properties that can be read and written
    as fields of the Account Action Ad Extension record in a bulk file.

    For more information, see Account Action Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass

class BulkCampaignActionAdExtension(_BulkCampaignAdExtensionAssociation):
    """ Represents a campaign level action ad extension.

    This class exposes properties that can be read and written
    as fields of the Campaign Action Ad Extension record in a bulk file.

    For more information, see Campaign Action Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass

class BulkAdGroupActionAdExtension(_BulkAdGroupAdExtensionAssociation):
    """ Represents an ad group level Action ad extension.

    This class exposes properties that can be read and written
    as fields of the Ad Group Action Ad Extension record in a bulk file.

    For more information, see Ad Group Action Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *

class BulkSharedNegativeSite(_SingleRecordBulkEntity):
    """ Represents a negative site.

    This class exposes the property :attr:`negative_site` that can be read and written as fields of the negative site record
    in a bulk file.

    For more information, see negative site at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, parent_id=None, status=None, negative_site=None):
        super(BulkSharedNegativeSite, self).__init__()

        self._parent_id = parent_id
        self._status = status
        self._negative_site = negative_site

    @property
    def parent_id(self):
        """ The identifier of the parent entity that contains the negative site.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._parent_id

    @parent_id.setter
    def parent_id(self, parent_id):
        self._parent_id = parent_id

    @property
    def status(self):
        """ The status of the negative site.

        Corresponds to the 'Status' field in the bulk file.

        :rtype: str
        """

        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    @property
    def negative_site(self):
        """ The NegativeSite Data Object of the Campaign Management Service.

        A subset of NegativeSite properties are available in the Ad Group record.
        For more information, see Ad Group at https://go.microsoft.com/fwlink/?linkid=846127.
        """
        return self._negative_site

    @negative_site.setter
    def negative_site(self, negative_site):
        self._negative_site = negative_site

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.negative_site.Id),
            csv_to_field=lambda c, v: setattr(c.negative_site, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(c, 'status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.parent_id),
            csv_to_field=lambda c, v: setattr(c, 'parent_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AccountPlacementListItemUrl,
            field_to_csv=lambda c: bulk_str(c.negative_site.Url),
            csv_to_field=lambda c, v: setattr(c.negative_site, 'Url', v if v else None)
        ),
    ]


    def process_mappings_from_row_values(self, row_values):
        self.negative_site = _CAMPAIGN_OBJECT_FACTORY_V13.create('NegativeSite')

        row_values.convert_to_entity(self, BulkSharedNegativeSite._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._negative_site, 'NegativeSite')
        self.convert_to_values(row_values, BulkSharedNegativeSite._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkSharedNegativeSite, self).read_additional_data(stream_reader)



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.campaign_criterion_status import CampaignCriterionStatus
from openapi_client.models.campaign.criterion import Criterion
from openapi_client.models.campaign.criterion_bid import CriterionBid
from openapi_client.models.campaign.criterion_cashback import CriterionCashback
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from typing_extensions import Self

class BiddableCampaignCriterion(BaseModel):
    """
    BiddableCampaignCriterion
    """ # noqa: E501
    criterion_bid: Optional[CriterionBid] = Field(default=None, alias="CriterionBid")
    criterion_cashback: Optional[CriterionCashback] = Field(default=None, alias="CriterionCashback")
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    campaign_id: Optional[StrictStr] = Field(default=None, alias="CampaignId")
    criterion: Optional[Criterion] = Field(default=None, alias="Criterion")
    forward_compatibility_map: Optional[List[Optional[KeyValuePairOfstringAndstring]]] = Field(default=None, alias="ForwardCompatibilityMap")
    status: Optional[CampaignCriterionStatus] = Field(default=None, alias="Status")
    type: Optional[StrictStr] = Field(default='BiddableCampaignCriterion', alias="Type")
    __properties: ClassVar[List[str]] = ["CriterionBid", "CriterionCashback", "Id", "CampaignId", "Criterion", "ForwardCompatibilityMap", "Status", "Type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	
    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())
	
    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of BiddableCampaignCriterion from a JSON string"""
        return cls.from_dict(json.loads(json_str))
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of criterion_bid
        if self.criterion_bid:
            _dict['CriterionBid'] = self.criterion_bid.to_dict()
        # override the default output from pydantic by calling `to_dict()` of criterion_cashback
        if self.criterion_cashback:
            _dict['CriterionCashback'] = self.criterion_cashback.to_dict()
        # override the default output from pydantic by calling `to_dict()` of criterion
        if self.criterion:
            _dict['Criterion'] = self.criterion.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in forward_compatibility_map (list)
        _items = []
        if self.forward_compatibility_map:
            for _item_forward_compatibility_map in self.forward_compatibility_map:
                if _item_forward_compatibility_map:
                    _items.append(_item_forward_compatibility_map.to_dict())
            _dict['ForwardCompatibilityMap'] = _items
        # set to None if criterion_bid (nullable) is None
        # and model_fields_set contains the field
        if self.criterion_bid is None and "criterion_bid" in self.model_fields_set:
            _dict['CriterionBid'] = None

        # set to None if criterion_cashback (nullable) is None
        # and model_fields_set contains the field
        if self.criterion_cashback is None and "criterion_cashback" in self.model_fields_set:
            _dict['CriterionCashback'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if campaign_id (nullable) is None
        # and model_fields_set contains the field
        if self.campaign_id is None and "campaign_id" in self.model_fields_set:
            _dict['CampaignId'] = None

        # set to None if criterion (nullable) is None
        # and model_fields_set contains the field
        if self.criterion is None and "criterion" in self.model_fields_set:
            _dict['Criterion'] = None

        # set to None if forward_compatibility_map (nullable) is None
        # and model_fields_set contains the field
        if self.forward_compatibility_map is None and "forward_compatibility_map" in self.model_fields_set:
            _dict['ForwardCompatibilityMap'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BiddableCampaignCriterion from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "CriterionBid": CriterionBid.from_dict(obj["CriterionBid"]) if obj.get("CriterionBid") is not None else None,
                        "CriterionCashback": CriterionCashback.from_dict(obj["CriterionCashback"]) if obj.get("CriterionCashback") is not None else None,
                        "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "CampaignId": obj.get("CampaignId") if obj.get("CampaignId") is not None else None,
                        "Criterion": Criterion.from_dict(obj["Criterion"]) if obj.get("Criterion") is not None else None,
                        "ForwardCompatibilityMap": [KeyValuePairOfstringAndstring.from_dict(_item) for _item in obj["ForwardCompatibilityMap"]] if obj.get("ForwardCompatibilityMap") is not None else None,
                        "Status": obj.get("Status") if obj.get("Status") is not None else None,
                        "Type": obj.get("Type") if obj.get("Type") is not None else 'BiddableCampaignCriterion'
        })
        return _obj



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.adinsight.broad_match_search_query_kpi import BroadMatchSearchQueryKPI
from typing_extensions import Self

class BroadMatchKeywordOpportunity(BaseModel):
    """
    BroadMatchKeywordOpportunity
    """ # noqa: E501
    click_share: Optional[StrictFloat] = Field(default=None, alias="ClickShare")
    average_ctr: Optional[StrictFloat] = Field(default=None, alias="AverageCTR")
    average_cpc: Optional[StrictFloat] = Field(default=None, alias="AverageCPC")
    reference_keyword_id: Optional[StrictStr] = Field(default=None, alias="ReferenceKeywordId")
    reference_keyword_bid: Optional[StrictFloat] = Field(default=None, alias="ReferenceKeywordBid")
    reference_keyword_match_type: Optional[StrictInt] = Field(default=None, alias="ReferenceKeywordMatchType")
    impression_share: Optional[StrictFloat] = Field(default=None, alias="ImpressionShare")
    search_query_kpis: Optional[List[Optional[BroadMatchSearchQueryKPI]]] = Field(default=None, alias="SearchQueryKPIs")
    ad_group_id: Optional[StrictStr] = Field(default=None, alias="AdGroupId")
    campaign_id: Optional[StrictStr] = Field(default=None, alias="CampaignId")
    ad_group_name: Optional[StrictStr] = Field(default=None, alias="AdGroupName")
    campaign_name: Optional[StrictStr] = Field(default=None, alias="CampaignName")
    match_type: Optional[StrictInt] = Field(default=None, alias="MatchType")
    suggested_keyword: Optional[StrictStr] = Field(default=None, alias="SuggestedKeyword")
    suggested_bid: Optional[StrictFloat] = Field(default=None, alias="SuggestedBid")
    monthly_searches: Optional[StrictStr] = Field(default=None, alias="MonthlySearches")
    competition: Optional[StrictFloat] = Field(default=None, alias="Competition")
    estimated_increase_in_clicks: Optional[StrictFloat] = Field(default=None, alias="EstimatedIncreaseInClicks")
    estimated_increase_in_cost: Optional[StrictFloat] = Field(default=None, alias="EstimatedIncreaseInCost")
    estimated_increase_in_impressions: Optional[StrictStr] = Field(default=None, alias="EstimatedIncreaseInImpressions")
    opportunity_key: Optional[StrictStr] = Field(default=None, alias="OpportunityKey")
    type: Optional[StrictStr] = Field(default='BroadMatchKeywordOpportunity', alias="Type")
    __properties: ClassVar[List[str]] = ["ClickShare", "AverageCTR", "AverageCPC", "ReferenceKeywordId", "ReferenceKeywordBid", "ReferenceKeywordMatchType", "ImpressionShare", "SearchQueryKPIs", "AdGroupId", "CampaignId", "AdGroupName", "CampaignName", "MatchType", "SuggestedKeyword", "SuggestedBid", "MonthlySearches", "Competition", "EstimatedIncreaseInClicks", "EstimatedIncreaseInCost", "EstimatedIncreaseInImpressions", "OpportunityKey", "Type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	
    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())
	
    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of BroadMatchKeywordOpportunity from a JSON string"""
        return cls.from_dict(json.loads(json_str))
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in search_query_kpis (list)
        _items = []
        if self.search_query_kpis:
            for _item_search_query_kpis in self.search_query_kpis:
                if _item_search_query_kpis:
                    _items.append(_item_search_query_kpis.to_dict())
            _dict['SearchQueryKpis'] = _items
        # set to None if click_share (nullable) is None
        # and model_fields_set contains the field
        if self.click_share is None and "click_share" in self.model_fields_set:
            _dict['ClickShare'] = None

        # set to None if average_ctr (nullable) is None
        # and model_fields_set contains the field
        if self.average_ctr is None and "average_ctr" in self.model_fields_set:
            _dict['AverageCTR'] = None

        # set to None if average_cpc (nullable) is None
        # and model_fields_set contains the field
        if self.average_cpc is None and "average_cpc" in self.model_fields_set:
            _dict['AverageCPC'] = None

        # set to None if reference_keyword_id (nullable) is None
        # and model_fields_set contains the field
        if self.reference_keyword_id is None and "reference_keyword_id" in self.model_fields_set:
            _dict['ReferenceKeywordId'] = None

        # set to None if reference_keyword_bid (nullable) is None
        # and model_fields_set contains the field
        if self.reference_keyword_bid is None and "reference_keyword_bid" in self.model_fields_set:
            _dict['ReferenceKeywordBid'] = None

        # set to None if reference_keyword_match_type (nullable) is None
        # and model_fields_set contains the field
        if self.reference_keyword_match_type is None and "reference_keyword_match_type" in self.model_fields_set:
            _dict['ReferenceKeywordMatchType'] = None

        # set to None if impression_share (nullable) is None
        # and model_fields_set contains the field
        if self.impression_share is None and "impression_share" in self.model_fields_set:
            _dict['ImpressionShare'] = None

        # set to None if search_query_kpis (nullable) is None
        # and model_fields_set contains the field
        if self.search_query_kpis is None and "search_query_kpis" in self.model_fields_set:
            _dict['SearchQueryKPIs'] = None

        # set to None if ad_group_id (nullable) is None
        # and model_fields_set contains the field
        if self.ad_group_id is None and "ad_group_id" in self.model_fields_set:
            _dict['AdGroupId'] = None

        # set to None if campaign_id (nullable) is None
        # and model_fields_set contains the field
        if self.campaign_id is None and "campaign_id" in self.model_fields_set:
            _dict['CampaignId'] = None

        # set to None if ad_group_name (nullable) is None
        # and model_fields_set contains the field
        if self.ad_group_name is None and "ad_group_name" in self.model_fields_set:
            _dict['AdGroupName'] = None

        # set to None if campaign_name (nullable) is None
        # and model_fields_set contains the field
        if self.campaign_name is None and "campaign_name" in self.model_fields_set:
            _dict['CampaignName'] = None

        # set to None if match_type (nullable) is None
        # and model_fields_set contains the field
        if self.match_type is None and "match_type" in self.model_fields_set:
            _dict['MatchType'] = None

        # set to None if suggested_keyword (nullable) is None
        # and model_fields_set contains the field
        if self.suggested_keyword is None and "suggested_keyword" in self.model_fields_set:
            _dict['SuggestedKeyword'] = None

        # set to None if suggested_bid (nullable) is None
        # and model_fields_set contains the field
        if self.suggested_bid is None and "suggested_bid" in self.model_fields_set:
            _dict['SuggestedBid'] = None

        # set to None if monthly_searches (nullable) is None
        # and model_fields_set contains the field
        if self.monthly_searches is None and "monthly_searches" in self.model_fields_set:
            _dict['MonthlySearches'] = None

        # set to None if competition (nullable) is None
        # and model_fields_set contains the field
        if self.competition is None and "competition" in self.model_fields_set:
            _dict['Competition'] = None

        # set to None if estimated_increase_in_clicks (nullable) is None
        # and model_fields_set contains the field
        if self.estimated_increase_in_clicks is None and "estimated_increase_in_clicks" in self.model_fields_set:
            _dict['EstimatedIncreaseInClicks'] = None

        # set to None if estimated_increase_in_cost (nullable) is None
        # and model_fields_set contains the field
        if self.estimated_increase_in_cost is None and "estimated_increase_in_cost" in self.model_fields_set:
            _dict['EstimatedIncreaseInCost'] = None

        # set to None if estimated_increase_in_impressions (nullable) is None
        # and model_fields_set contains the field
        if self.estimated_increase_in_impressions is None and "estimated_increase_in_impressions" in self.model_fields_set:
            _dict['EstimatedIncreaseInImpressions'] = None

        # set to None if opportunity_key (nullable) is None
        # and model_fields_set contains the field
        if self.opportunity_key is None and "opportunity_key" in self.model_fields_set:
            _dict['OpportunityKey'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BroadMatchKeywordOpportunity from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "ClickShare": obj.get("ClickShare") if obj.get("ClickShare") is not None else None,
                        "AverageCTR": obj.get("AverageCTR") if obj.get("AverageCTR") is not None else None,
                        "AverageCPC": obj.get("AverageCPC") if obj.get("AverageCPC") is not None else None,
                        "ReferenceKeywordId": obj.get("ReferenceKeywordId") if obj.get("ReferenceKeywordId") is not None else None,
                        "ReferenceKeywordBid": obj.get("ReferenceKeywordBid") if obj.get("ReferenceKeywordBid") is not None else None,
                        "ReferenceKeywordMatchType": obj.get("ReferenceKeywordMatchType") if obj.get("ReferenceKeywordMatchType") is not None else None,
                        "ImpressionShare": obj.get("ImpressionShare") if obj.get("ImpressionShare") is not None else None,
                        "SearchQueryKPIs": [BroadMatchSearchQueryKPI.from_dict(_item) for _item in obj["SearchQueryKpis"]] if obj.get("SearchQueryKpis") is not None else None,
                        "AdGroupId": obj.get("AdGroupId") if obj.get("AdGroupId") is not None else None,
                        "CampaignId": obj.get("CampaignId") if obj.get("CampaignId") is not None else None,
                        "AdGroupName": obj.get("AdGroupName") if obj.get("AdGroupName") is not None else None,
                        "CampaignName": obj.get("CampaignName") if obj.get("CampaignName") is not None else None,
                        "MatchType": obj.get("MatchType") if obj.get("MatchType") is not None else None,
                        "SuggestedKeyword": obj.get("SuggestedKeyword") if obj.get("SuggestedKeyword") is not None else None,
                        "SuggestedBid": obj.get("SuggestedBid") if obj.get("SuggestedBid") is not None else None,
                        "MonthlySearches": obj.get("MonthlySearches") if obj.get("MonthlySearches") is not None else None,
                        "Competition": obj.get("Competition") if obj.get("Competition") is not None else None,
                        "EstimatedIncreaseInClicks": obj.get("EstimatedIncreaseInClicks") if obj.get("EstimatedIncreaseInClicks") is not None else None,
                        "EstimatedIncreaseInCost": obj.get("EstimatedIncreaseInCost") if obj.get("EstimatedIncreaseInCost") is not None else None,
                        "EstimatedIncreaseInImpressions": obj.get("EstimatedIncreaseInImpressions") if obj.get("EstimatedIncreaseInImpressions") is not None else None,
                        "OpportunityKey": obj.get("OpportunityKey") if obj.get("OpportunityKey") is not None else None,
                        "Type": obj.get("Type") if obj.get("Type") is not None else 'BroadMatchKeywordOpportunity'
        })
        return _obj



# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AccountFinancialStatus(str, Enum):
    PROPOSED = 'Proposed'

    PENDINGCREDITCHECK = 'PendingCreditCheck'

    CLEARFINANCIALSTATUS = 'ClearFinancialStatus'

    SOLDTOONLY = 'SoldToOnly'

    CREDITWARNING = 'CreditWarning'

    HOLD = 'Hold'

    WRITEOFF = 'WriteOff'

    TAXONHOLD = 'TaxOnHold'

    USERHOLD = 'UserHold'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AccountFinancialStatus from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.adinsight.budget_point_type import BudgetPointType
from typing_extensions import Self

class BudgetPoint(BaseModel):
    """
    BudgetPoint
    """ # noqa: E501
    budget_amount: Optional[StrictFloat] = Field(default=None, alias="BudgetAmount")
    budget_point_type: Optional[BudgetPointType] = Field(default=None, alias="BudgetPointType")
    estimated_weekly_clicks: Optional[StrictFloat] = Field(default=None, alias="EstimatedWeeklyClicks")
    estimated_weekly_cost: Optional[StrictFloat] = Field(default=None, alias="EstimatedWeeklyCost")
    estimated_weekly_impressions: Optional[StrictFloat] = Field(default=None, alias="EstimatedWeeklyImpressions")
    __properties: ClassVar[List[str]] = ["BudgetAmount", "BudgetPointType", "EstimatedWeeklyClicks", "EstimatedWeeklyCost", "EstimatedWeeklyImpressions"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if budget_amount (nullable) is None
        # and model_fields_set contains the field
        if self.budget_amount is None and "budget_amount" in self.model_fields_set:
            _dict['BudgetAmount'] = None

        # set to None if budget_point_type (nullable) is None
        # and model_fields_set contains the field
        if self.budget_point_type is None and "budget_point_type" in self.model_fields_set:
            _dict['BudgetPointType'] = None

        # set to None if estimated_weekly_clicks (nullable) is None
        # and model_fields_set contains the field
        if self.estimated_weekly_clicks is None and "estimated_weekly_clicks" in self.model_fields_set:
            _dict['EstimatedWeeklyClicks'] = None

        # set to None if estimated_weekly_cost (nullable) is None
        # and model_fields_set contains the field
        if self.estimated_weekly_cost is None and "estimated_weekly_cost" in self.model_fields_set:
            _dict['EstimatedWeeklyCost'] = None

        # set to None if estimated_weekly_impressions (nullable) is None
        # and model_fields_set contains the field
        if self.estimated_weekly_impressions is None and "estimated_weekly_impressions" in self.model_fields_set:
            _dict['EstimatedWeeklyImpressions'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BudgetPoint from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "BudgetAmount": obj.get("BudgetAmount") if obj.get("BudgetAmount") is not None else None,
                        "BudgetPointType": obj.get("BudgetPointType") if obj.get("BudgetPointType") is not None else None,
                        "EstimatedWeeklyClicks": obj.get("EstimatedWeeklyClicks") if obj.get("EstimatedWeeklyClicks") is not None else None,
                        "EstimatedWeeklyCost": obj.get("EstimatedWeeklyCost") if obj.get("EstimatedWeeklyCost") is not None else None,
                        "EstimatedWeeklyImpressions": obj.get("EstimatedWeeklyImpressions") if obj.get("EstimatedWeeklyImpressions") is not None else None
        })
        return _obj



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.adinsight.day_month_and_year import DayMonthAndYear
from typing_extensions import Self

class GetHistoricalSearchCountRequest(BaseModel):
    """
    GetHistoricalSearchCountRequest
    """ # noqa: E501
    keywords: Optional[List[StrictStr]] = Field(default=None, alias="Keywords")
    language: Optional[StrictStr] = Field(default=None, alias="Language")
    publisher_countries: Optional[List[StrictStr]] = Field(default=None, alias="PublisherCountries")
    start_date: Optional[DayMonthAndYear] = Field(default=None, alias="StartDate")
    end_date: Optional[DayMonthAndYear] = Field(default=None, alias="EndDate")
    time_period_rollup: Optional[StrictStr] = Field(default=None, alias="TimePeriodRollup")
    devices: Optional[List[StrictStr]] = Field(default=None, alias="Devices")
    __properties: ClassVar[List[str]] = ["Keywords", "Language", "PublisherCountries", "StartDate", "EndDate", "TimePeriodRollup", "Devices"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of start_date
        if self.start_date:
            _dict['StartDate'] = self.start_date.to_dict()
        # override the default output from pydantic by calling `to_dict()` of end_date
        if self.end_date:
            _dict['EndDate'] = self.end_date.to_dict()
        # set to None if keywords (nullable) is None
        # and model_fields_set contains the field
        if self.keywords is None and "keywords" in self.model_fields_set:
            _dict['Keywords'] = None

        # set to None if language (nullable) is None
        # and model_fields_set contains the field
        if self.language is None and "language" in self.model_fields_set:
            _dict['Language'] = None

        # set to None if publisher_countries (nullable) is None
        # and model_fields_set contains the field
        if self.publisher_countries is None and "publisher_countries" in self.model_fields_set:
            _dict['PublisherCountries'] = None

        # set to None if start_date (nullable) is None
        # and model_fields_set contains the field
        if self.start_date is None and "start_date" in self.model_fields_set:
            _dict['StartDate'] = None

        # set to None if end_date (nullable) is None
        # and model_fields_set contains the field
        if self.end_date is None and "end_date" in self.model_fields_set:
            _dict['EndDate'] = None

        # set to None if time_period_rollup (nullable) is None
        # and model_fields_set contains the field
        if self.time_period_rollup is None and "time_period_rollup" in self.model_fields_set:
            _dict['TimePeriodRollup'] = None

        # set to None if devices (nullable) is None
        # and model_fields_set contains the field
        if self.devices is None and "devices" in self.model_fields_set:
            _dict['Devices'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetHistoricalSearchCountRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Keywords": obj.get("Keywords"),
                        "Language": obj.get("Language") if obj.get("Language") is not None else None,
                        "PublisherCountries": obj.get("PublisherCountries"),
                        "StartDate": DayMonthAndYear.from_dict(obj["StartDate"]) if obj.get("StartDate") is not None else None,
                        "EndDate": DayMonthAndYear.from_dict(obj["EndDate"]) if obj.get("EndDate") is not None else None,
                        "TimePeriodRollup": obj.get("TimePeriodRollup") if obj.get("TimePeriodRollup") is not None else None,
                        "Devices": obj.get("Devices")
        })
        return _obj



from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *

class BulkSharedSite(_SingleRecordBulkEntity):
    """ Represents a site.

    This class exposes the property :attr:`site` that can be read and written as fields of the site record
    in a bulk file.

    For more information, see site at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, parent_id=None, url=None, status=None, site=None):
        super(BulkSharedSite, self).__init__()

        self._parent_id = parent_id
        self._url = url
        self._status = status
        self._site = site

    @property
    def parent_id(self):
        """ The identifier of the parent entity that contains the site.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._parent_id

    @parent_id.setter
    def parent_id(self, parent_id):
        self._parent_id = parent_id

    @property
    def url(self):
        """ The url of a website.

        Corresponds to the 'Website' field in the bulk file.

        :rtype: str
        """

        return self._url

    @url.setter
    def url(self, url):
        self._url = url

    @property
    def status(self):
        """ The status of the site.

        Corresponds to the 'Status' field in the bulk file.

        :rtype: str
        """

        return self._status

    @status.setter
    def status(self, status):
        self._status = status

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.site.Id),
            csv_to_field=lambda c, v: setattr(c.site, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.status),
            csv_to_field=lambda c, v: setattr(c, 'status', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.parent_id),
            csv_to_field=lambda c, v: setattr(c, 'parent_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AccountPlacementListItemUrl,
            field_to_csv=lambda c: bulk_str(c.site.Url),
            csv_to_field=lambda c, v: setattr(c.site, 'Url', v if v else None)
        ),
    ]


    def process_mappings_from_row_values(self, row_values):
        self.site = _CAMPAIGN_OBJECT_FACTORY_V13.create('Site')

        row_values.convert_to_entity(self, BulkSharedSite._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self._site, 'Site')
        self.convert_to_values(row_values, BulkSharedSite._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(BulkSharedSite, self).read_additional_data(stream_reader)



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class PerformanceMaxSetting(BaseModel):
    """
    PerformanceMaxSetting
    """ # noqa: E501
    final_url_expansion_opt_out: Optional[StrictBool] = Field(default=None, alias="FinalUrlExpansionOptOut")
    page_feed_ids: Optional[List[StrictStr]] = Field(default=None, alias="PageFeedIds")
    auto_generated_text_opt_out: Optional[StrictBool] = Field(default=None, alias="AutoGeneratedTextOptOut")
    auto_generated_image_opt_out: Optional[StrictBool] = Field(default=None, alias="AutoGeneratedImageOptOut")
    cost_per_sale_opt_out: Optional[StrictBool] = Field(default=None, alias="CostPerSaleOptOut")
    type: Optional[StrictStr] = Field(default='PerformanceMaxSetting', alias="Type")
    __properties: ClassVar[List[str]] = ["FinalUrlExpansionOptOut", "PageFeedIds", "AutoGeneratedTextOptOut", "AutoGeneratedImageOptOut", "CostPerSaleOptOut", "Type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	
    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())
	
    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PerformanceMaxSetting from a JSON string"""
        return cls.from_dict(json.loads(json_str))
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if final_url_expansion_opt_out (nullable) is None
        # and model_fields_set contains the field
        if self.final_url_expansion_opt_out is None and "final_url_expansion_opt_out" in self.model_fields_set:
            _dict['FinalUrlExpansionOptOut'] = None

        # set to None if page_feed_ids (nullable) is None
        # and model_fields_set contains the field
        if self.page_feed_ids is None and "page_feed_ids" in self.model_fields_set:
            _dict['PageFeedIds'] = None

        # set to None if auto_generated_text_opt_out (nullable) is None
        # and model_fields_set contains the field
        if self.auto_generated_text_opt_out is None and "auto_generated_text_opt_out" in self.model_fields_set:
            _dict['AutoGeneratedTextOptOut'] = None

        # set to None if auto_generated_image_opt_out (nullable) is None
        # and model_fields_set contains the field
        if self.auto_generated_image_opt_out is None and "auto_generated_image_opt_out" in self.model_fields_set:
            _dict['AutoGeneratedImageOptOut'] = None

        # set to None if cost_per_sale_opt_out (nullable) is None
        # and model_fields_set contains the field
        if self.cost_per_sale_opt_out is None and "cost_per_sale_opt_out" in self.model_fields_set:
            _dict['CostPerSaleOptOut'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PerformanceMaxSetting from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "FinalUrlExpansionOptOut": obj.get("FinalUrlExpansionOptOut") if obj.get("FinalUrlExpansionOptOut") is not None else None,
                        "PageFeedIds": obj.get("PageFeedIds"),
                        "AutoGeneratedTextOptOut": obj.get("AutoGeneratedTextOptOut") if obj.get("AutoGeneratedTextOptOut") is not None else None,
                        "AutoGeneratedImageOptOut": obj.get("AutoGeneratedImageOptOut") if obj.get("AutoGeneratedImageOptOut") is not None else None,
                        "CostPerSaleOptOut": obj.get("CostPerSaleOptOut") if obj.get("CostPerSaleOptOut") is not None else None,
                        "Type": obj.get("Type") if obj.get("Type") is not None else 'PerformanceMaxSetting'
        })
        return _obj



# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.reporting.account_status_report_filter import AccountStatusReportFilter
from openapi_client.models.reporting.ad_group_status_report_filter import AdGroupStatusReportFilter
from openapi_client.models.reporting.ad_status_report_filter import AdStatusReportFilter
from openapi_client.models.reporting.ad_type_report_filter import AdTypeReportFilter
from openapi_client.models.reporting.asset_group_status_report_filter import AssetGroupStatusReportFilter
from openapi_client.models.reporting.campaign_status_report_filter import CampaignStatusReportFilter
from openapi_client.models.reporting.language_report_filter import LanguageReportFilter
from typing_extensions import Self

class ProductSearchQueryPerformanceReportFilter(BaseModel):
    """
    ProductSearchQueryPerformanceReportFilter
    """ # noqa: E501
    account_status: Optional[AccountStatusReportFilter] = Field(default=None, alias="AccountStatus")
    ad_status: Optional[AdStatusReportFilter] = Field(default=None, alias="AdStatus")
    ad_type: Optional[AdTypeReportFilter] = Field(default=None, alias="AdType")
    campaign_status: Optional[CampaignStatusReportFilter] = Field(default=None, alias="CampaignStatus")
    exclude_zero_clicks: Optional[StrictBool] = Field(default=None, alias="ExcludeZeroClicks")
    language: Optional[LanguageReportFilter] = Field(default=None, alias="Language")
    search_queries: Optional[List[StrictStr]] = Field(default=None, alias="SearchQueries")
    ad_group_status: Optional[AdGroupStatusReportFilter] = Field(default=None, alias="AdGroupStatus")
    asset_group_status: Optional[AssetGroupStatusReportFilter] = Field(default=None, alias="AssetGroupStatus")
    __properties: ClassVar[List[str]] = ["AccountStatus", "AdStatus", "AdType", "CampaignStatus", "ExcludeZeroClicks", "Language", "SearchQueries", "AdGroupStatus", "AssetGroupStatus"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if account_status (nullable) is None
        # and model_fields_set contains the field
        if self.account_status is None and "account_status" in self.model_fields_set:
            _dict['AccountStatus'] = None

        # set to None if ad_status (nullable) is None
        # and model_fields_set contains the field
        if self.ad_status is None and "ad_status" in self.model_fields_set:
            _dict['AdStatus'] = None

        # set to None if ad_type (nullable) is None
        # and model_fields_set contains the field
        if self.ad_type is None and "ad_type" in self.model_fields_set:
            _dict['AdType'] = None

        # set to None if campaign_status (nullable) is None
        # and model_fields_set contains the field
        if self.campaign_status is None and "campaign_status" in self.model_fields_set:
            _dict['CampaignStatus'] = None

        # set to None if exclude_zero_clicks (nullable) is None
        # and model_fields_set contains the field
        if self.exclude_zero_clicks is None and "exclude_zero_clicks" in self.model_fields_set:
            _dict['ExcludeZeroClicks'] = None

        # set to None if language (nullable) is None
        # and model_fields_set contains the field
        if self.language is None and "language" in self.model_fields_set:
            _dict['Language'] = None

        # set to None if search_queries (nullable) is None
        # and model_fields_set contains the field
        if self.search_queries is None and "search_queries" in self.model_fields_set:
            _dict['SearchQueries'] = None

        # set to None if ad_group_status (nullable) is None
        # and model_fields_set contains the field
        if self.ad_group_status is None and "ad_group_status" in self.model_fields_set:
            _dict['AdGroupStatus'] = None

        # set to None if asset_group_status (nullable) is None
        # and model_fields_set contains the field
        if self.asset_group_status is None and "asset_group_status" in self.model_fields_set:
            _dict['AssetGroupStatus'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ProductSearchQueryPerformanceReportFilter from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "AccountStatus": obj.get("AccountStatus") if obj.get("AccountStatus") is not None else None,
                        "AdStatus": obj.get("AdStatus") if obj.get("AdStatus") is not None else None,
                        "AdType": obj.get("AdType") if obj.get("AdType") is not None else None,
                        "CampaignStatus": obj.get("CampaignStatus") if obj.get("CampaignStatus") is not None else None,
                        "ExcludeZeroClicks": obj.get("ExcludeZeroClicks") if obj.get("ExcludeZeroClicks") is not None else None,
                        "Language": obj.get("Language") if obj.get("Language") is not None else None,
                        "SearchQueries": obj.get("SearchQueries"),
                        "AdGroupStatus": obj.get("AdGroupStatus") if obj.get("AdGroupStatus") is not None else None,
                        "AssetGroupStatus": obj.get("AssetGroupStatus") if obj.get("AssetGroupStatus") is not None else None
        })
        return _obj



# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class SearchInsightPerformanceReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    ACCOUNTNUMBER = 'AccountNumber'

    ACCOUNTID = 'AccountId'

    TIMEPERIOD = 'TimePeriod'

    CAMPAIGNNAME = 'CampaignName'

    CAMPAIGNID = 'CampaignId'

    ADGROUPNAME = 'AdGroupName'

    ADGROUPID = 'AdGroupId'

    CAMPAIGNSTATUS = 'CampaignStatus'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    CTR = 'Ctr'

    SPEND = 'Spend'

    SEARCHQUERY = 'SearchQuery'

    CONVERSIONS = 'Conversions'

    CONVERSIONRATE = 'ConversionRate'

    REVENUE = 'Revenue'

    RETURNONADSPEND = 'ReturnOnAdSpend'

    REVENUEPERCONVERSION = 'RevenuePerConversion'

    ACCOUNTSTATUS = 'AccountStatus'

    ADGROUPSTATUS = 'AdGroupStatus'

    CUSTOMERID = 'CustomerId'

    CUSTOMERNAME = 'CustomerName'

    ASSETGROUPID = 'AssetGroupId'

    ASSETGROUPNAME = 'AssetGroupName'

    ASSETGROUPSTATUS = 'AssetGroupStatus'

    SEARCHCATEGORY = 'SearchCategory'

    SEARCHVOLUME = 'SearchVolume'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SearchInsightPerformanceReportColumn from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.product_condition import ProductCondition
from openapi_client.models.campaign.product_partition_type import ProductPartitionType
from typing_extensions import Self

class ProductPartition(BaseModel):
    """
    ProductPartition
    """ # noqa: E501
    parent_criterion_id: Optional[StrictStr] = Field(default=None, alias="ParentCriterionId")
    condition: Optional[ProductCondition] = Field(default=None, alias="Condition")
    partition_type: Optional[ProductPartitionType] = Field(default=None, alias="PartitionType")
    type: Optional[StrictStr] = Field(default='ProductPartition', alias="Type")
    __properties: ClassVar[List[str]] = ["ParentCriterionId", "Condition", "PartitionType", "Type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	
    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())
	
    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ProductPartition from a JSON string"""
        return cls.from_dict(json.loads(json_str))
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of condition
        if self.condition:
            _dict['Condition'] = self.condition.to_dict()
        # set to None if parent_criterion_id (nullable) is None
        # and model_fields_set contains the field
        if self.parent_criterion_id is None and "parent_criterion_id" in self.model_fields_set:
            _dict['ParentCriterionId'] = None

        # set to None if condition (nullable) is None
        # and model_fields_set contains the field
        if self.condition is None and "condition" in self.model_fields_set:
            _dict['Condition'] = None

        # set to None if partition_type (nullable) is None
        # and model_fields_set contains the field
        if self.partition_type is None and "partition_type" in self.model_fields_set:
            _dict['PartitionType'] = None

        # set to None if type (nullable) is None
        # and model_fields_set contains the field
        if self.type is None and "type" in self.model_fields_set:
            _dict['Type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ProductPartition from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "ParentCriterionId": obj.get("ParentCriterionId") if obj.get("ParentCriterionId") is not None else None,
                        "Condition": ProductCondition.from_dict(obj["Condition"]) if obj.get("Condition") is not None else None,
                        "PartitionType": obj.get("PartitionType") if obj.get("PartitionType") is not None else None,
                        "Type": obj.get("Type") if obj.get("Type") is not None else 'ProductPartition'
        })
        return _obj



# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class TravelQueryInsightReportColumn(str, Enum):
    SEARCHQUERY = 'SearchQuery'

    COUNTRY = 'Country'

    REGION = 'Region'

    METROAREA = 'MetroArea'

    CITY = 'City'

    POSTALCODE = 'PostalCode'

    L1VERTICAL = 'L1Vertical'

    L2VERTICAL = 'L2Vertical'

    L3VERTICAL = 'L3Vertical'

    POSCOUNTRY = 'POSCountry'

    POSREGION = 'POSRegion'

    POSCITY = 'POSCity'

    BRANDIMPRESSIONS = 'BrandImpressions'

    BRANDCLICKS = 'BrandClicks'

    BRANDSPEND = 'BrandSpend'

    MSAIMPRESSIONS = 'MSAImpressions'

    MSACLICKS = 'MSAClicks'

    MSASPEND = 'MSASpend'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of TravelQueryInsightReportColumn from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class BrandKitFont(BaseModel):
    """
    BrandKitFont
    """ # noqa: E501
    typeface: Optional[StrictStr] = Field(default=None, alias="Typeface")
    weight: Optional[StrictStr] = Field(default=None, alias="Weight")
    text_asset_type: Optional[StrictStr] = Field(default=None, alias="TextAssetType")
    __properties: ClassVar[List[str]] = ["Typeface", "Weight", "TextAssetType"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if typeface (nullable) is None
        # and model_fields_set contains the field
        if self.typeface is None and "typeface" in self.model_fields_set:
            _dict['Typeface'] = None

        # set to None if weight (nullable) is None
        # and model_fields_set contains the field
        if self.weight is None and "weight" in self.model_fields_set:
            _dict['Weight'] = None

        # set to None if text_asset_type (nullable) is None
        # and model_fields_set contains the field
        if self.text_asset_type is None and "text_asset_type" in self.model_fields_set:
            _dict['TextAssetType'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BrandKitFont from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Typeface": obj.get("Typeface") if obj.get("Typeface") is not None else None,
                        "Weight": obj.get("Weight") if obj.get("Weight") is not None else None,
                        "TextAssetType": obj.get("TextAssetType") if obj.get("TextAssetType") is not None else None
        })
        return _obj



# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ShareOfVoiceReportColumn(str, Enum):
    ACCOUNTNAME = 'AccountName'

    TIMEPERIOD = 'TimePeriod'

    CAMPAIGNNAME = 'CampaignName'

    ADGROUPNAME = 'AdGroupName'

    KEYWORD = 'Keyword'

    DELIVEREDMATCHTYPE = 'DeliveredMatchType'

    BIDMATCHTYPE = 'BidMatchType'

    LANGUAGE = 'Language'

    ACCOUNTNUMBER = 'AccountNumber'

    ACCOUNTID = 'AccountId'

    KEYWORDID = 'KeywordId'

    ADGROUPID = 'AdGroupId'

    CAMPAIGNID = 'CampaignId'

    IMPRESSIONS = 'Impressions'

    CLICKS = 'Clicks'

    CTR = 'Ctr'

    AVERAGECPC = 'AverageCpc'

    SPEND = 'Spend'

    AVERAGEPOSITION = 'AveragePosition'

    IMPRESSIONSHAREPERCENT = 'ImpressionSharePercent'

    IMPRESSIONLOSTTOBUDGETPERCENT = 'ImpressionLostToBudgetPercent'

    IMPRESSIONLOSTTORANKAGGPERCENT = 'ImpressionLostToRankAggPercent'

    CURRENTMAXCPC = 'CurrentMaxCpc'

    QUALITYSCORE = 'QualityScore'

    EXPECTEDCTR = 'ExpectedCtr'

    ADRELEVANCE = 'AdRelevance'

    LANDINGPAGEEXPERIENCE = 'LandingPageExperience'

    CONVERSIONS = 'Conversions'

    CONVERSIONRATE = 'ConversionRate'

    COSTPERCONVERSION = 'CostPerConversion'

    ADDISTRIBUTION = 'AdDistribution'

    CLICKSHAREPERCENT = 'ClickSharePercent'

    DEVICETYPE = 'DeviceType'

    NETWORK = 'Network'

    ACCOUNTSTATUS = 'AccountStatus'

    CAMPAIGNSTATUS = 'CampaignStatus'

    ADGROUPSTATUS = 'AdGroupStatus'

    KEYWORDSTATUS = 'KeywordStatus'

    BIDSTRATEGYTYPE = 'BidStrategyType'

    KEYWORDLABELS = 'KeywordLabels'

    EXACTMATCHIMPRESSIONSHAREPERCENT = 'ExactMatchImpressionSharePercent'

    TOPIMPRESSIONSHARELOSTTORANKPERCENT = 'TopImpressionShareLostToRankPercent'

    TOPIMPRESSIONSHARELOSTTOBUDGETPERCENT = 'TopImpressionShareLostToBudgetPercent'

    ABSOLUTETOPIMPRESSIONSHARELOSTTORANKPERCENT = 'AbsoluteTopImpressionShareLostToRankPercent'

    ABSOLUTETOPIMPRESSIONSHARELOSTTOBUDGETPERCENT = 'AbsoluteTopImpressionShareLostToBudgetPercent'

    ABSOLUTETOPIMPRESSIONSHAREPERCENT = 'AbsoluteTopImpressionSharePercent'

    TOPIMPRESSIONSHAREPERCENT = 'TopImpressionSharePercent'

    ABSOLUTETOPIMPRESSIONRATEPERCENT = 'AbsoluteTopImpressionRatePercent'

    TOPIMPRESSIONRATEPERCENT = 'TopImpressionRatePercent'

    BASECAMPAIGNID = 'BaseCampaignId'

    ALLCONVERSIONS = 'AllConversions'

    ALLCONVERSIONRATE = 'AllConversionRate'

    ALLCOSTPERCONVERSION = 'AllCostPerConversion'

    VIEWTHROUGHCONVERSIONS = 'ViewThroughConversions'

    GOAL = 'Goal'

    GOALTYPE = 'GoalType'

    AVERAGECPM = 'AverageCpm'

    CONVERSIONSQUALIFIED = 'ConversionsQualified'

    ALLCONVERSIONSQUALIFIED = 'AllConversionsQualified'

    VIEWTHROUGHCONVERSIONSQUALIFIED = 'ViewThroughConversionsQualified'

    VIEWTHROUGHREVENUE = 'ViewThroughRevenue'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ShareOfVoiceReportColumn from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.customer.lcid import LCID
from typing_extensions import Self

class UserInvitation(BaseModel):
    """
    UserInvitation
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    first_name: Optional[StrictStr] = Field(default=None, alias="FirstName")
    last_name: Optional[StrictStr] = Field(default=None, alias="LastName")
    email: Optional[StrictStr] = Field(default=None, alias="Email")
    customer_id: Optional[StrictStr] = Field(default=None, alias="CustomerId")
    role_id: Optional[StrictInt] = Field(default=None, alias="RoleId")
    account_ids: Optional[List[StrictStr]] = Field(default=None, alias="AccountIds")
    expiration_date: Optional[datetime] = Field(default=None, alias="ExpirationDate")
    lcid: Optional[LCID] = Field(default=None, alias="Lcid")
    __properties: ClassVar[List[str]] = ["Id", "FirstName", "LastName", "Email", "CustomerId", "RoleId", "AccountIds", "ExpirationDate", "Lcid"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if first_name (nullable) is None
        # and model_fields_set contains the field
        if self.first_name is None and "first_name" in self.model_fields_set:
            _dict['FirstName'] = None

        # set to None if last_name (nullable) is None
        # and model_fields_set contains the field
        if self.last_name is None and "last_name" in self.model_fields_set:
            _dict['LastName'] = None

        # set to None if email (nullable) is None
        # and model_fields_set contains the field
        if self.email is None and "email" in self.model_fields_set:
            _dict['Email'] = None

        # set to None if customer_id (nullable) is None
        # and model_fields_set contains the field
        if self.customer_id is None and "customer_id" in self.model_fields_set:
            _dict['CustomerId'] = None

        # set to None if role_id (nullable) is None
        # and model_fields_set contains the field
        if self.role_id is None and "role_id" in self.model_fields_set:
            _dict['RoleId'] = None

        # set to None if account_ids (nullable) is None
        # and model_fields_set contains the field
        if self.account_ids is None and "account_ids" in self.model_fields_set:
            _dict['AccountIds'] = None

        # set to None if expiration_date (nullable) is None
        # and model_fields_set contains the field
        if self.expiration_date is None and "expiration_date" in self.model_fields_set:
            _dict['ExpirationDate'] = None

        # set to None if lcid (nullable) is None
        # and model_fields_set contains the field
        if self.lcid is None and "lcid" in self.model_fields_set:
            _dict['Lcid'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UserInvitation from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "FirstName": obj.get("FirstName") if obj.get("FirstName") is not None else None,
                        "LastName": obj.get("LastName") if obj.get("LastName") is not None else None,
                        "Email": obj.get("Email") if obj.get("Email") is not None else None,
                        "CustomerId": obj.get("CustomerId") if obj.get("CustomerId") is not None else None,
                        "RoleId": obj.get("RoleId") if obj.get("RoleId") is not None else None,
                        "AccountIds": obj.get("AccountIds"),
                        "ExpirationDate": obj.get("ExpirationDate") if obj.get("ExpirationDate") is not None else None,
                        "Lcid": obj.get("Lcid") if obj.get("Lcid") is not None else None
        })
        return _obj



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.adinsight.currency_code import CurrencyCode
from typing_extensions import Self

class BidLandscapePoint(BaseModel):
    """
    BidLandscapePoint
    """ # noqa: E501
    bid: Optional[StrictFloat] = Field(default=None, alias="Bid")
    clicks: Optional[StrictFloat] = Field(default=None, alias="Clicks")
    impressions: Optional[StrictStr] = Field(default=None, alias="Impressions")
    top_impressions: Optional[StrictStr] = Field(default=None, alias="TopImpressions")
    currency_code: Optional[CurrencyCode] = Field(default=None, alias="CurrencyCode")
    cost: Optional[StrictFloat] = Field(default=None, alias="Cost")
    marginal_cpc: Optional[StrictFloat] = Field(default=None, alias="MarginalCPC")
    __properties: ClassVar[List[str]] = ["Bid", "Clicks", "Impressions", "TopImpressions", "CurrencyCode", "Cost", "MarginalCPC"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if bid (nullable) is None
        # and model_fields_set contains the field
        if self.bid is None and "bid" in self.model_fields_set:
            _dict['Bid'] = None

        # set to None if clicks (nullable) is None
        # and model_fields_set contains the field
        if self.clicks is None and "clicks" in self.model_fields_set:
            _dict['Clicks'] = None

        # set to None if impressions (nullable) is None
        # and model_fields_set contains the field
        if self.impressions is None and "impressions" in self.model_fields_set:
            _dict['Impressions'] = None

        # set to None if top_impressions (nullable) is None
        # and model_fields_set contains the field
        if self.top_impressions is None and "top_impressions" in self.model_fields_set:
            _dict['TopImpressions'] = None

        # set to None if currency_code (nullable) is None
        # and model_fields_set contains the field
        if self.currency_code is None and "currency_code" in self.model_fields_set:
            _dict['CurrencyCode'] = None

        # set to None if cost (nullable) is None
        # and model_fields_set contains the field
        if self.cost is None and "cost" in self.model_fields_set:
            _dict['Cost'] = None

        # set to None if marginal_cpc (nullable) is None
        # and model_fields_set contains the field
        if self.marginal_cpc is None and "marginal_cpc" in self.model_fields_set:
            _dict['MarginalCPC'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BidLandscapePoint from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Bid": obj.get("Bid") if obj.get("Bid") is not None else None,
                        "Clicks": obj.get("Clicks") if obj.get("Clicks") is not None else None,
                        "Impressions": obj.get("Impressions") if obj.get("Impressions") is not None else None,
                        "TopImpressions": obj.get("TopImpressions") if obj.get("TopImpressions") is not None else None,
                        "CurrencyCode": obj.get("CurrencyCode") if obj.get("CurrencyCode") is not None else None,
                        "Cost": obj.get("Cost") if obj.get("Cost") is not None else None,
                        "MarginalCPC": obj.get("MarginalCPC") if obj.get("MarginalCPC") is not None else None
        })
        return _obj
